<?php
// $Id: pageroute.page.inc,v 1.1.2.3 2008/11/14 10:33:30 tauran Exp $

/**
 * @file
 * Page base type
 */
 
abstract class pageroute_page {
  var $title;
  var $options;
  var $route;
  var $name;
  var $weight;
  var $module;
  
  /**
   * Returns the form to display the page type.
   * @param $args
   *    Can be used to customize the form.
   *  @return
   *    The form to display.
   */  
  abstract public function set_form(&$form, &$form_state, &$args);
  
  /**
   *  Include stuff your page type needs.
   */  
  abstract protected function set_up();
  
  public function pageroute_page($page) {
    $this->load($page);
  }
  
  public function load($page) {
    $this->title = $page->title;
    $this->options = $page->options;
    $this->name = $page->name;
    $this->type = $page->type;
    $this->weight = $page->weight;
    $this->module = $page->module;
    
    $this->route = &$page->route;
  }
  
  /**
   *  Return the default action, if a cancel button is clicked.
   *  @return
   *    PAGEROUTE_CURRENT: stay on current page
   *    PAGEROUTE_FORWARD: go to next page
   *    PAGEROUTE_BACK: go to previous page
   */  
  public function get_cancel_target() {
    return PAGEROUTE_CURRENT;
  }
  
  public static function get_object($page_data) {
    $type = $page_data->type;
    $bases = pageroute_get_types('base');   
    
    // Include needed type!
    include_once(drupal_get_path('module', $page_data->module) .'/'. $page_data->module .'.page_'. $page_data->type .'.inc');
  
    $page_class = $page_data->module .'_page_'. $page_data->type;
    
    $page = new $page_class($page_data);
    
    $page->set_up();
  
    return $page;
  }
  
  /**
   * These form fields are quite common for node forms, so page
   * types, which are also displaying node forms can use it.
   * @param $delete
   *    Wheter the "delete checkbox" should be added.
   */
  public static function node_ui($page, &$form, $delete = TRUE) {
    $form['options']['preview'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display preview button'),
      '#default_value' => isset($page->options['preview']) ? $page->options['preview'] : 0,
      '#weight' => 5,        
    );
    $form['options']['submit'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display submit button'),
      '#default_value' => isset($page->options['submit']) ? $page->options['submit'] : 1,
      '#weight' => 5,        
    );
    if ($delete) {
      $form['options']['nodelete'] = array(
        '#type' => 'checkbox',
        '#title' => t('Never display the delete button'),
        '#default_value' => isset($page->options['nodelete']) ? $page->options['nodelete'] : 1,
        '#weight' => 5,        
      );
    }
  }

  public function configure_form(&$form) {
    if (isset($this->options['preview']) && !$this->options['preview']) {
      unset($form['buttons']['preview']);
    }

    if (isset($this->options['submit']) && !$this->options['submit']) {
      unset($form['buttons']['submit']);
    }

    if (isset($this->options['nodelete']) && $this->options['nodelete']) {
      unset($form['buttons']['delete']);
    }
  }
  
  /**
   * Handles all button clicks from your specific form.
   *  Set $args['target'] ( = $form_state['storage']['args']['target']) to specify the what to do:
   *    PAGEROUTE_CURRENT: stay on current page
   *    PAGEROUTE_FORWARD: go to next page
   *    PAGEROUTE_BACK: go to previous page
   *  To customize get_form(...) set $args['todo'] ( = $form_state['storage']['args']['todo']).
   */  
  public static function form_submitted(&$form_state) { }
  
  /**
   * Describes the page type.
   *  @return
   *    array('name' => t('Human readable page type name'));
   */  
  public abstract static function info();
  
  /**
   * Provides a help text for the pageroute user interface.
   *  @return
   *    t('Help text.')
   */  
  public abstract static function help();

  public static function get_default_submit_handler($form) { }
  public static function get_default_validate_handler($form) { }
}
